#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const dotenv_1 = __importDefault(require("dotenv"));
const Leetcode = __importStar(require("./leetcode"));
const Problem = __importStar(require("./problem"));
const path_1 = __importDefault(require("path"));
const log_1 = __importDefault(require("./log"));
const ErrorMsg = __importStar(require("./errorMsg"));
const interface_1 = require("./interface");
const leetcode_api_typescript_1 = require("leetcode-api-typescript");
const fs_1 = __importDefault(require("fs"));
async function addProblemProgram(program) {
    const getDefaultProblemSlug = () => {
        let slug;
        slug = process.env.INIT_CWD?.split(path_1.default.sep).slice(-1)[0];
        slug = slug.split(".").slice(-1)[0];
        return slug;
    };
    const getDefaultDst = () => {
        return process.env.INIT_CWD;
    };
    const getDefaultLangSlug = () => {
        return leetcode_api_typescript_1.LangSlug.cpp;
    };
    const makeProblemCommand = (name) => {
        const subProgram = program.command(name);
        subProgram.description("some operations related to the problem");
        const newCommand = subProgram.command("new <slug>");
        const pullCommand = subProgram.command("pull");
        const codeCommand = subProgram.command("code");
        const submitCommand = subProgram.command("submit");
        const newAction = async (slug) => {
            const options = newCommand.opts();
            const { dst } = options;
            try {
                await Problem.New(slug, dst);
            }
            catch (err) {
                log_1.default.Error(err);
            }
        };
        const pullAction = async () => {
            const options = pullCommand.opts();
            let { slug, dst } = options;
            try {
                if (!slug) {
                    slug = getDefaultProblemSlug();
                }
                if (!dst) {
                    dst = getDefaultDst();
                }
                await Problem.Pull(slug, dst);
            }
            catch (err) {
                log_1.default.Error(err);
            }
        };
        const codeAction = async () => {
            const options = codeCommand.opts();
            let { slug, dst, langSlug, fileName } = options;
            try {
                if (!process.env.LEETCODE_TEMPLATE_PATH) {
                    throw new Error(`${ErrorMsg.EnvironmentVariableDoesNotExist} [envName=LEETCODE_TEMPLATE_PATH]`);
                }
                if (!slug) {
                    slug = getDefaultProblemSlug();
                }
                if (!dst) {
                    dst = getDefaultDst();
                }
                if (!langSlug) {
                    langSlug = getDefaultLangSlug();
                }
                else {
                    langSlug = leetcode_api_typescript_1.LangSlug[langSlug];
                }
                fileName = (0, interface_1.SolutionFileName)(langSlug, fileName);
                const codeTemplateFileName = (0, interface_1.CodeTemplateFileName)(langSlug);
                const templateContent = fs_1.default
                    .readFileSync(path_1.default.join(process.env.LEETCODE_TEMPLATE_PATH, codeTemplateFileName))
                    .toString();
                await Problem.Code(slug, langSlug, templateContent, path_1.default.join(dst, fileName));
            }
            catch (err) {
                log_1.default.Error(err);
            }
        };
        const submitAction = async () => {
            const options = submitCommand.opts();
            let { slug, dst, langSlug, fileName } = options;
            try {
                if (!slug) {
                    slug = getDefaultProblemSlug();
                }
                if (!dst) {
                    dst = getDefaultDst();
                }
                if (!langSlug) {
                    langSlug = getDefaultLangSlug();
                }
                else {
                    langSlug = leetcode_api_typescript_1.LangSlug[langSlug];
                }
                fileName = (0, interface_1.SolutionFileName)(langSlug, fileName);
                const codePath = path_1.default.join(dst, fileName);
                log_1.default.Info(`codePath: ${codePath}`);
                const code = fs_1.default.readFileSync(codePath).toString();
                await Problem.Submit(slug, langSlug, code);
            }
            catch (err) {
                console.log(err);
            }
        };
        newCommand.option("-d, --dst <string>").action(newAction);
        pullCommand
            .option("-s --slug <string>")
            .option("-d, --dst <string>")
            .action(pullAction);
        codeCommand
            .option("-s --slug <string>")
            .option("-d --dst <string>")
            .option("--langSlug <string>")
            .option("--fileName <string>")
            .action(codeAction);
        submitCommand
            .option("-s --slug <string>")
            .option("-d --dst <string>")
            .option("--langSlug <string>")
            .option("--fileName <string>")
            .action(submitAction);
    };
    makeProblemCommand("problem");
    makeProblemCommand("p");
}
(async () => {
    dotenv_1.default.config();
    await Leetcode.Login();
    const program = new commander_1.Command();
    program.description("A cli tool to enjoy leetcode!");
    addProblemProgram(program);
    await program.parseAsync(process.argv);
})();
//# sourceMappingURL=index.js.map