"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Submit = exports.Code = exports.Pull = exports.New = void 0;
const leetcode_api_typescript_1 = require("leetcode-api-typescript");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const ErrorMsg = __importStar(require("./errorMsg"));
const log_1 = __importDefault(require("./log"));
const interface_1 = require("./interface");
const utils_1 = require("./utils");
async function New(slug, dst) {
    const problem = await leetcode_api_typescript_1.Problem.build(slug);
    const targetDirName = `${problem.id}.${problem.slug}`;
    if (dst) {
        dst = path.join(dst, targetDirName);
    }
    else {
        dst = path.join(process.env.INIT_CWD ?? "", targetDirName);
    }
    if (fs.existsSync(dst)) {
        throw new Error(`${ErrorMsg.DestinationAlreadyExists}. [dst=${dst}]`);
    }
    fs.mkdirSync(dst, { recursive: true });
    await getStatement(problem, dst);
}
exports.New = New;
async function Pull(slug, dst) {
    const problem = await leetcode_api_typescript_1.Problem.build(slug);
    await getStatement(problem, dst);
}
exports.Pull = Pull;
async function Code(slug, langSlug, template, dst) {
    const problem = await leetcode_api_typescript_1.Problem.build(slug);
    for (const codeSnippet of problem.codeSnippets ?? []) {
        if (codeSnippet.langSlug === leetcode_api_typescript_1.LangSlug[langSlug]) {
            if (fs.existsSync(dst)) {
                throw new Error(`${ErrorMsg.DestinationAlreadyExists}. [dst=${dst}]`);
            }
            fs.writeFileSync(dst, template.replace((0, interface_1.CodeTemplateReplaceContent)(langSlug), codeSnippet.code));
            log_1.default.Info(`generate code successfully. [dst=${dst}]`);
            return;
        }
    }
    throw new Error(`${ErrorMsg.UnsupportedLangslugType}. [langSlug=${langSlug}]`);
}
exports.Code = Code;
async function Submit(slug, langSlug, code) {
    const problem = await leetcode_api_typescript_1.Problem.build(slug);
    const submission = await problem.submit(langSlug, code);
    log_1.default.Info(`submit successfully. [slug=${slug}, langSlug=${leetcode_api_typescript_1.LangSlug[langSlug]}]`);
    for (let i = 0; i < 3; i++) {
        await submission.detail();
        if (submission.statusDisplay === "") {
            (0, utils_1.Sleep)(500);
            continue;
        }
    }
    log_1.default.Info(`id: ${submission.id}
lang: ${submission.lang}
runtime: ${submission.runtime}
memory: ${submission.memory}
status: ${submission.statusDisplay}
timestamp: ${submission.timestamp}
submissionUrl: ${submission.getSubmissionUrl()}`);
}
exports.Submit = Submit;
async function getStatement(problem, dst) {
    const statement = {
        en_US: "statement.en_US.md",
        zh_CN: "statement.zh_CN.md",
    };
    const statementPath = (() => {
        const statementPath = {};
        for (const k in statement) {
            statementPath[k] = path.join(dst, statement[k]);
        }
        return statementPath;
    })();
    for (const k in statementPath) {
        if (fs.existsSync(statementPath[k])) {
            log_1.default.Warn(`${ErrorMsg.DestinationAlreadyExists}, will be removed. [dst=${statementPath[k]}`);
            fs.rmSync(statementPath[k]);
        }
    }
    const getProblemLink = (baseUrl, slug, title) => {
        return `<p><strong><a href="${baseUrl}/problems/${slug}/" target="_blank" rel="noopener noreferrer">${title}</a></strong></p>\n\n`;
    };
    if (problem.content) {
        fs.writeFileSync(statementPath["en_US"], getProblemLink(leetcode_api_typescript_1.Config.uri.us.base, problem.slug, problem.title) + problem.content);
        log_1.default.Info(`pull statement successfully. [path=${statementPath["en_US"]}]`);
    }
    if (problem.translatedContent) {
        fs.writeFileSync(statementPath["zh_CN"], getProblemLink(leetcode_api_typescript_1.Config.uri.cn.base, problem.slug, problem.translatedTitle) + problem.translatedContent);
        log_1.default.Info(`pull statement successfully. [path=${statementPath["zh_CN"]}]`);
    }
}
//# sourceMappingURL=problem.js.map